<?php
/****************************************************************************/
/* MakeArt.php - (c) Peardox 2022                                           */
/*                                                                          */
/* A simple script to automate the creation of AI Art using the registered  */
/* version of Deep Art Effects                                              */
/*                                                                          */
/* Usage : makeart <output_path> <filename_prefix> <from_number> <how_many> */
/*                                                                          */
/* Sample Usage                                                             */
/*                                                                          */
/* /php/php makeart.php art ai 1 100                                        */
/*                                                                          */
/* The above command will make a directory named art and create 100 AI art  */
/* images in that directory named ai-000001.jpg thru ai-000100.jpg          */
/*                                                                          */
/* When specifying the output directory you may create multiple levels at   */
/* once by specifying multiple directories - e.g. dir1\dir2\dir3 will make  */
/* the requested directories and output to the full path specified          */
/*                                                                          */
/* The filename prefix and directory names can only include alpha-numeric   */
/* (a-z, A-Z, 0-9), stop (.), minus (-) and underscore(_) characters        */
/*                                                                          */
/* The output filename generated will consist of the filename_prefix        */
/* followed by a dash/minus (-) and six zero padded digits of type jpg      */ 
/*                                                                          */
/* If you want to output pngs simply change the extension on line 31        */
/****************************************************************************/

// Output file type - jpg is smaller, png is fairly redundant in this case really
$ftype = "jpg";

// This function gets the current time in microseconds since 01/01/1970 00:00:00
function microtime_float() {
  list($usec, $sec) = explode(" ", microtime());
  return ((float)$usec + (float)$sec);
}


// Check filename is valid
function check_filename($name) {
	if (!preg_match('/^[\w\-._]+$/', $name)) {
		echo "File names may only include the characters [A-Z][a-z][0-9] ._ and -\n";
		return false;
	} else {
		return true;
	}
}

// Check pathname is valid
function check_pathname($name) {
	// Silently exchange DOS path delimeter for Unix version
	$name = str_replace("\\", "/", $name);
	if (!preg_match('/^[\/\w\-._]+$/', $name)) {
		echo "Path names may only include the characters [A-Z][a-z][0-9] \\/._ and -\n";
		return false;
	} else {
		return true;
	}
}

// Allow the script to run for as long as it wants
set_time_limit(0);

// Check DAE is in current directory
if(file_exists("dae.exe")) {
	$dae = "dae.exe"; // I can't be assed typing the full name so I always rename it dae
} else if(file_exists("DeepArtEffectsCli.exe")) {
	$dae = "DeepArtEffectsCli.exe";
} else {
	die("Cant find Deep Art Effects CLI\n");
}

// Total runtime
$total_time = 0;

// Start time
$start = microtime_float();

// Initialise the sequential numbering for output files (in case not enough parameters are supplied)
$outpath = "";
$fnprefix = "";
$from = 0;
$howmany = 0;

// Fetch the command line parameters
if($argc === 5) {
	$outpath = $argv[1];
	$fnprefix = $argv[2];
	$from = (integer) $argv[3];
	$howmany = (integer) $argv[4];
}

// If the command line parameters look bad fail with help
if((check_pathname($outpath) === false) || (check_filename($fnprefix) === false) || ($from === 0) || ($howmany === 0) || ($argc !== 5)) {
	// End run with helpful message
	die("makeart <output_path> <filename_prefix> <from_number> <how_many>\n        e.g. makeart art ai_image 1 10");
}

// Counter for current image
$image_index = 0;

// If the output path doesn't exist then create it (can make multiple directory levels all at once)
if(!file_exists($outpath)) {
	mkdir($outpath, 0700, true);
}

// Make sure the first and last files don't exists already to deter accidentally overwriting existing files
if(file_exists($outpath . "/" . $fnprefix . "-" . sprintf("%06d", $from) . "." . $ftype) ||
   file_exists($outpath . "/" . $fnprefix . "-" . sprintf("%06d", $from + $howmany - 1) . "." . $ftype)) {
    die("At least one of the requested output files already exist");
}

// We now want to make $howmany images starting with image $fnprefix-$from
for($i=$from; $i<=($from + $howmany - 1); $i++) {
	// Construct the command line
	$cmd = ".\\" . $dae . " generate -name artwork -output " . $outpath . "/" . $fnprefix . "-" . sprintf("%06d", $i) . "." . $ftype;
	// Run the command -  output all DAE stuff to out-makeart.log
	if(!shell_exec($cmd . " 2>out-makeart.log")) {
		// Oops - nothing good, obviously
		die("Something went wrong");
	}

	// Calculate some stats so the user knows how things are going
	$elapsed = microtime_float() - $start;
	$start = microtime_float();
	$total_time += $elapsed;
	$gentime = $total_time / ($image_index + 1);
	$eta = round(($howmany - $image_index) * $gentime);


	// Tell the user what's going on
	echo "Created " . 
		$fnprefix . "-" . sprintf("%06d", $i) . "." . $ftype .
		", time = " . 
		sprintf("%0.3f", $elapsed) . 
		", avg = " . 
		sprintf("%0.3f", $gentime) . 
		" sec, Rate = " . 
		sprintf("%0.3f", 60 / $gentime) . 
		" per min, " .
		sprintf("%d", 3600 / $gentime) . 
		" per hour, ETA " .
		sprintf('%d:%02d:%02d', 
				floor($eta / 3600),
				floor($eta / 60) % 60, 
				$eta % 60) .
		", " . ($howmany - $image_index) . " left\n";
		
	// Next image...
	$image_index++;
}

// Output the final statistics
$total_time = round($total_time);

echo "\nCreated $image_index images in $outpath which took " .
	sprintf('%d:%02d:%02d', 
		floor($total_time / 3600),
		floor($total_time / 60) % 60, 
		$total_time % 60) . 
	"\n\n";
